package com.winhc.bigdata.spark.jobs

import com.winhc.bigdata.spark.utils.{EsUtils, SparkUtils}
import org.apache.commons.lang3.StringUtils

import scala.collection.JavaConverters._

/**
 * @Author: XuJiakai
 * @Date: 2020/6/5 14:28
 * @Description:
 */
object CompanyIndexSave2Es {
  val pattern = "[^\\u4e00-\\u9fa50-9a-zA-Z]".r

  case class CompanyName(show: String, value: String)

  case class CompanyDoc(cname: CompanyName, current_id: String = null, history_name: Seq[CompanyName] = null, company_type: String)

  def getEsDoc(cid: String, cname: String, other_id_name: scala.collection.Map[String, String], new_cid: String, company_type: String): (String, CompanyDoc) = {
    var history_name: Seq[CompanyName] = null
    if (other_id_name != null) {
      history_name = other_id_name
        .filterKeys(!_.equals(new_cid))
        .filterKeys(!_.equals(cid))
        .values
        .map(getCompanyName)
        .toSeq
      if (history_name.isEmpty) {
        history_name = null
      }
    }
    (cid, CompanyDoc(getCompanyName(cname), if (cid.equals(new_cid)) null else new_cid, history_name, company_type))
  }

  private def getOtherIdName(str: String): scala.collection.Map[String, String] = {
    if (str == null) {
      return null
    }
    str.split("\002").map(s => {
      val sp = s.split("\001")
      (sp(0), sp(1))
    }).toMap
  }


  private def getCompanyName(name: String): CompanyName = if (StringUtils.isEmpty(name)) null else CompanyName(name, pattern replaceAllIn(name, ""))

  def main(args: Array[String]): Unit = {
    val map = EsUtils.getEsConfigMap

    val company_name_mapping = "winhc_eci_dev.company_name_mapping_pro_v2"

    val spark = SparkUtils.InitEnv("CompanyIndexSave2Es", map)
    import org.elasticsearch.spark._
    import spark.implicits._
    val df = spark.sql(s"select cid,cname,other_id_name,new_cid,company_type from $company_name_mapping")
    df.map(r => {
      val cid = r.getLong(0).toString
      val cname = r.getString(1)
      val other_id_name = getOtherIdName(r.getString(2))
      val new_cid = r.getString(3)
      val company_type = r.getString(4)
      getEsDoc(cid, cname, other_id_name, new_cid, company_type)
    }).rdd.saveToEsWithMeta("winhc-company-v2/company")
    spark.stop()
  }
}
