package com.winhc.bigdata.spark.utils

import java.util.Date

import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * 增量cid转换
 */
case class CompanyIncrForCidUtils(s: SparkSession,
                                  project: String, //表所在工程名
                                  tableName: String, //表名（不加前后辍）
                                  dupliCols: Seq[String] // 去重列
                                 ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s

  def calc(): Unit = {
    val inc_ods_company = s"${project}.inc_ods_company" //每日公司基本信息增量
    val ads_company_tb = s"${project}.ads_${tableName}"
    val inc_ods_company_tb = s"${project}.inc_ods_$tableName"
    val target_inc_ods_company_tb = s"${project}.inc_ads_$tableName"


    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)

    //val lastDs = BaseUtil.getPartion("winhc_eci_dev.ads_company_icp", spark)
    val firstDs = BaseUtil.getFirstPartion(inc_ods_company, spark)

    //table字段
    val columns: Seq[String] = spark.table(ads_company_tb).schema.map(_.name).filter(s => {
      !s.equals("ds") && !s.equals("cid") && !s.equals("new_cid") && !s.equals("rowkey")
    })

    sql(
      s"""
         |SELECT  cid,current_cid as new_cid
         |FROM    ${inc_ods_company}
         |WHERE   ds >= ${firstDs}
         |AND     cid IS NOT NULL
         |AND     current_cid IS NOT NULL
         |GROUP BY cid,current_cid
         |""".stripMargin).cache().createOrReplaceTempView("mapping")

    sql(
      s"""
         |INSERT OVERWRITE TABLE ${target_inc_ods_company_tb} PARTITION(ds=$firstDs)
         |SELECT  CONCAT_WS('_',new_cid,id) AS rowkey
         |        ,flag
         |        ,new_cid
         |        ,cid
         |        ,${columns.mkString(",")}
         |FROM    (
         |            SELECT  "0" AS flag
         |                    ,a.new_cid
         |                    ,b.cid
         |                    ,${columns.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY ${dupliCols.mkString(",")} ORDER BY update_time DESC ) num
         |            FROM    mapping a
         |            JOIN    (
         |                        SELECT  new_cid AS cid
         |                                ,${columns.mkString(",")}
         |                        FROM    ${ads_company_tb}
         |                        WHERE   ds >= ${firstDs}
         |                        UNION ALL
         |                        SELECT  new_cid AS cid
         |                                ,${columns.mkString(",")}
         |                        FROM    ${target_inc_ods_company_tb}
         |                        WHERE   ds >= ${firstDs}
         |                    ) b
         |            ON      a.cid = b.cid
         |            UNION ALL
         |            SELECT  "1" AS flag
         |                    ,coalesce(b.new_cid,a.cid) new_cid
         |                    ,cast(a.cid as string) as cid
         |                    ,${columns.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY ${dupliCols.mkString(",")} ORDER BY update_time DESC ) num
         |            FROM    ${inc_ods_company_tb} a
         |            LEFT JOIN mapping b
         |            ON      a.cid = b.cid
         |            WHERE   a.ds >= ${firstDs}
         |            AND     a.cid IS NOT NULL
         |        ) d
         |WHERE   num = 1
         |""".stripMargin)

    val colsTotal = columns ++ Seq("new_cid")

    MaxComputer2Phoenix(
      spark,
      colsTotal,
      target_inc_ods_company_tb,
      tableName,
      firstDs,
      Seq("new_cid","id")
    ).syn()

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }


}