package com.winhc.bigdata.spark.utils

import java.util.Date

import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * π
 * 增量cids转换
 */

case class CompanyIncrForCidsUtils(s: SparkSession,
                                   project: String, //表所在工程名
                                   mainTableName: String, //主表名（不加前辍）
                                   sublistTableName: String, //子表（不加前辍）
                                   dupliCols: Seq[String] // 去重列
                                  ) extends LoggingUtils {
  @(transient@getter) val spark: SparkSession = s

  def calc(): Unit = {
    val inc_ods_company = s"${project}.inc_ods_company" //每日公司基本信息增量
    val ads_company_tb = s"${project}.ads_$mainTableName" //存量ads主表数据
    val ads_company_tb_list = s"${project}.ads_$sublistTableName" //存量子表数据 用于读取表字段
    val inc_ods_company_tb = s"${project}.inc_ods_$mainTableName" //增量数据ods 主表
    val target_inc_ads_company_tb = s"${project}.inc_ads_$mainTableName" //增量数据ads 主表
    val target_inc_ads_company_tb_list = s"${project}.inc_ads_$sublistTableName" //增量数据ads 子表


    val sublistTableFieldName = spark.table(ads_company_tb_list).columns.filter(s => {
      !s.equals("ds") && !s.equals("new_cid") && !s.equals("rowkey") && !s.equals("cids") && !s.equals("new_cids")
    }).seq

    println(s"${this.getClass.getSimpleName} calc start! " + new Date().toString)

    val firstDs = BaseUtil.getFirstPartion("winhc_eci_dev.inc_ods_company", spark)

    //table字段
    val columns: Seq[String] = spark.table(ads_company_tb).schema.map(_.name).filter(s => {
      !s.equals("ds") && !s.equals("cid") && !s.equals("new_cid") && !s.equals("rowkey") && !s.equals("cids") && !s.equals("new_cids")
    })

    //mapping 映射关系
    sql(
      s"""
         |SELECT  cid,current_cid as new_cid
         |FROM    ${inc_ods_company}
         |WHERE   ds >= ${firstDs}
         |AND     cid IS NOT NULL
         |AND     current_cid IS NOT NULL
         |GROUP BY cid,current_cid
         |""".stripMargin).cache().createOrReplaceTempView("mapping")

    //增量打平
    sql(
      s"""
         |SELECT  *
         |FROM    ${inc_ods_company_tb} a
         |LATERAL VIEW explode(split(cids,';')) b AS cid
         |WHERE   ds >= ${firstDs}
         |AND     cids IS NOT NULL
         |AND     trim(cids) <> ''
         |""".stripMargin).createOrReplaceTempView("incr_tb")


    //替换cid,去重,复制老数据
    val df1 = sql(
      s"""
         |INSERT OVERWRITE TABLE  $target_inc_ads_company_tb_list PARTITION(ds='$firstDs')
         |SELECT  CONCAT_WS('_',new_cid,id) AS rowkey
         |        ,"0" as flag
         |        ,CAST(new_cid as string) AS new_cid
         |        ,${sublistTableFieldName.mkString(",")}
         |FROM    (
         |        SELECT
         |                *
         |                ,ROW_NUMBER() OVER (PARTITION BY ${dupliCols.mkString(",")} ORDER BY update_time DESC ) num
         |        FROM    (
         |                SELECT
         |                        c.*
         |                        ,coalesce(d.new_cid,c.cid) AS new_cid
         |                FROM    incr_tb c
         |                LEFT JOIN mapping d
         |                ON      c.cid = d.cid
         |                ) e
         |        ) f
         |WHERE   num =1
         |UNION ALL
         |SELECT  CONCAT_WS('_',new_cid,id) AS rowkey
         |        ,"1" as flag
         |        ,CAST(new_cid as string) AS new_cid
         |        ,${sublistTableFieldName.mkString(",")}
         |FROM    (
         |            SELECT  a.new_cid
         |                    ,${columns.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY ${dupliCols.mkString(",")} ORDER BY update_time DESC ) num
         |            FROM    mapping a
         |            JOIN    (
         |                        SELECT  new_cid AS cid
         |                                ,${columns.mkString(",")}
         |                        FROM    ${target_inc_ads_company_tb_list}
         |                        WHERE   ds >= ${firstDs}
         |                        UNION ALL
         |                        SELECT  new_cid AS cid
         |                                ,${columns.mkString(",")}
         |                        FROM    ${ads_company_tb_list}
         |                        WHERE   ds >= ${firstDs}
         |                    ) b
         |            ON      a.cid = b.cid
         |        ) c
         |WHERE   num = 1
         |""".stripMargin)

    //主表按照id去重落库
    sql(
      s"""
         |INSERT OVERWRITE TABLE  $target_inc_ads_company_tb PARTITION(ds='$firstDs')
         |SELECT  cids,${columns.mkString(",")}
         |FROM    (
         |            SELECT  cids,${columns.mkString(",")}
         |                    ,ROW_NUMBER() OVER (PARTITION BY id ORDER BY update_time DESC ) num
         |            FROM    ${inc_ods_company_tb}
         |            WHERE   ds >= ${firstDs}
         |            AND     cids IS NOT NULL
         |            AND     trim(cids) <> ''
         |        ) a
         |WHERE   num = 1
         |""".stripMargin)

    //同步增量索引表数据
    val colsList = sublistTableFieldName ++ Seq("new_cid")
    MaxComputer2Phoenix(
      spark,
      colsList,
      target_inc_ads_company_tb_list,
      sublistTableName,
      firstDs,
      Seq("new_cid","id")
    ).syn()

    //同步增量主表数据
    val cols = columns ++ Seq("cids")
    MaxComputer2Phoenix(
      spark,
      cols,
      target_inc_ads_company_tb,
      mainTableName,
      firstDs,
      Seq("id")
    ).syn()

    println(s"${this.getClass.getSimpleName} calc end! " + new Date().toString)
  }
}