package com.winhc.bigdata.spark.utils

import java.util.Properties

import com.winhc.bigdata.spark.const.{BaseConst, EnvConst}

/**
 * @Author: XuJiakai
 * @Date: 2020/6/3 18:09
 * @Description:
 */
object PhoenixUtil {
  private val DB_PHOENIX_DRIVER = BaseConst.DB_PHOENIX_DRIVER
  private val DB_PHOENIX_USER = ""
  private val DB_PHOENIX_PASS = ""
  private val DB_PHOENIX_FETCHSIZE = "10000"

  def getPhoenixProperties: Properties = {
    val connProp = new Properties
    connProp.put("driver", DB_PHOENIX_DRIVER)
    connProp.put("user", DB_PHOENIX_USER)
    connProp.put("password", DB_PHOENIX_PASS)
    connProp.put("fetchsize", DB_PHOENIX_FETCHSIZE)
    connProp
  }


  def getPhoenixJDBCUrl: String = {
    val queryServerAddress: String = EnvConst.getEnv().getValue("phoenix.address")
    val url = "jdbc:phoenix:thin:url=" + queryServerAddress + ";serialization=PROTOBUF"
    url
  }

  def getPhoenixTempView(tempViewTableName: String, phoenixTableName: String, fetchsize: Int = 1000): String =
    s"""
       |CREATE TABLE $tempViewTableName USING org.apache.spark.sql.jdbc
       |OPTIONS (
       |  'driver' 'org.apache.phoenix.queryserver.client.Driver',
       |  'url' '${getPhoenixJDBCUrl}',
       |  'dbtable' '$phoenixTableName',
       |  'fetchsize' '$fetchsize'
       |)
       |""".stripMargin


  def getPhoenixOptions(tableName: String): Map[String, String] = {
    Map("table" -> tableName, "zkUrl" -> EnvConst.getEnv().getValue("zk.address"))
  }
}
